// The contents of this file are subject to the Mozilla Public License Version
// 1.1
//(the "License"); you may not use this file except in compliance with the
//License. You may obtain a copy of the License at http://www.mozilla.org/MPL/
//
//Software distributed under the License is distributed on an "AS IS" basis,
//WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
//for the specific language governing rights and
//limitations under the License.
//
//The Original Code is "The Columba Project"
//
//The Initial Developers of the Original Code are Frederik Dietz and Timo
// Stich.
//Portions created by Frederik Dietz and Timo Stich are Copyright (C) 2003.
//
//All Rights Reserved.
package org.columba.mail.folderoptions;

import org.columba.core.config.DefaultItem;
import org.columba.core.config.IDefaultItem;
import org.columba.core.xml.XmlElement;
import org.columba.mail.folder.IMailbox;
import org.columba.mail.gui.frame.MailFrameMediator;
import org.columba.mail.gui.frame.TableViewOwner;
import org.columba.mail.gui.table.IMessageNode;
import org.columba.mail.gui.table.TableController;
import org.columba.mail.gui.table.TableView;
import org.columba.mail.gui.table.model.HeaderTableModel;

/**
 * Handles selecting message after folder selection changes.
 * <p>
 * This implementation remembers the the selected message, and tries to reselect
 * it again. As default fall back it selects the first or last message,
 * depending on the sorting order.
 * 
 * @author fdietz, waffel
 */
public class SelectionOptionsPlugin extends AbstractFolderOptionsPlugin {

	/**
	 * Constructor
	 * 
	 * @param mediator
	 *            mail frame mediator
	 */
	public SelectionOptionsPlugin(MailFrameMediator mediator) {
		super("selection", "SelectionOptions", mediator);
	}

	/**
	 * 
	 * Save currently selected message.
	 * 
	 * @see org.columba.mail.folderoptions.AbstractFolderOptionsPlugin#saveOptionsToXml(IMailbox)
	 */
	public void saveOptionsToXml(IMailbox folder) {
		XmlElement parent = getConfigNode(folder);
		IDefaultItem item = new DefaultItem(parent);

		TableController tableController = ((TableController)((TableViewOwner) getMediator()).getTableController());

		if (tableController.getSelectedNodes() == null)
			return;

		if (tableController.getSelectedNodes().length == 0)
			return;

		IMessageNode node = tableController.getSelectedNodes()[0];
		if ((node != null) && (folder != null))
			folder.setLastSelection(node.getUid());
	}

	/**
	 * Restore selection.
	 * 
	 * @see org.columba.mail.folderoptions.AbstractFolderOptionsPlugin#loadOptionsFromXml(IMailbox)
	 */
	public void loadOptionsFromXml(IMailbox folder) {
		XmlElement parent = getConfigNode(folder);
		IDefaultItem item = new DefaultItem(parent);

		TableController tableController = ((TableController)((TableViewOwner) getMediator()).getTableController());

		TableView view = tableController.getView();

		// should we re-use the last remembered selection?
		boolean remember = item.getBooleanWithDefault("remember_last_selection", true);

		// sorting order
		boolean ascending = tableController.getTableModelSorter()
				.getSortingOrder();

		// row count
		int row = view.getTree().getRowCount();

		// row count == 0 --> empty table
		if (row == 0) {
			//  clear message viewer
			///tableController.valueChanged(new ListSelectionEvent(this,-1,-1,false));
			tableController.clearSelection();
			return;
		}

		// if the last selection for the current folder is null, then we show
		// the
		// first/last message in the table and scroll to it.
		if ((!remember) || (folder.getLastSelection() == null)) {
			// changing the selection to the first/last row based on ascending
			// state
			Object uid = null;

			if (ascending) {
				uid = view.selectLastRow();
			} else {
				uid = view.selectFirstRow();
			}

			// no messages in this folder
			if (uid == null) {
				return;
			}

		} else {

			// if a lastSelection for this folder is set
			// getting the last selected uid
			Object[] lastSelUids = { folder.getLastSelection() };

			// no messages in this folder
			if (lastSelUids[0] == null) {
				return;
			}

			Object uid = lastSelUids[0];

			// this message doesn't exit in this folder anymore
			if (((HeaderTableModel)tableController.getHeaderTableModel()).getMessageNode(uid) == null) {

				if (ascending) {
					uid = view.selectLastRow();
				} else {
					uid = view.selectFirstRow();
				}

			} else {

				// selecting the message
				tableController.setSelected(new Object[] { uid });
			}

		}
	}

	/**
	 * @see org.columba.mail.folderoptions.AbstractFolderOptionsPlugin#createDefaultElement()
	 */
	public XmlElement createDefaultElement(boolean global) {
		XmlElement parent = super.createDefaultElement(global);
		parent.addAttribute("remember_last_selection", "true");

		return parent;
	}
}