//The contents of this file are subject to the Mozilla Public License Version 1.1
//(the "License"); you may not use this file except in compliance with the 
//License. You may obtain a copy of the License at http://www.mozilla.org/MPL/
//
//Software distributed under the License is distributed on an "AS IS" basis,
//WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License 
//for the specific language governing rights and
//limitations under the License.
//
//The Original Code is "The Columba Project"
//
//The Initial Developers of the Original Code are Frederik Dietz and Timo Stich.
//Portions created by Frederik Dietz and Timo Stich are Copyright (C) 2003. 
//
//All Rights Reserved.
package org.columba.mail.main;

import java.awt.BorderLayout;
import java.util.logging.Logger;

import javax.swing.JCheckBox;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.OptionBuilder;
import org.columba.core.backgroundtask.BackgroundTaskManager;
import org.columba.core.backgroundtask.TaskInterface;
import org.columba.core.config.DefaultItem;
import org.columba.core.config.IDefaultItem;
import org.columba.core.gui.frame.DefaultContainer;
import org.columba.core.gui.frame.FrameModel;
import org.columba.core.gui.util.MultiLineLabel;
import org.columba.core.main.ColumbaCmdLineParser;
import org.columba.core.main.ConnectionStateImpl;
import org.columba.core.main.IComponentPlugin;
import org.columba.core.main.Main;
import org.columba.core.plugin.PluginHandlerNotFoundException;
import org.columba.core.plugin.PluginLoadingFailedException;
import org.columba.core.plugin.PluginManager;
import org.columba.core.pluginhandler.ActionPluginHandler;
import org.columba.core.services.ServiceManager;
import org.columba.core.shutdown.ShutdownManager;
import org.columba.core.util.GlobalResourceLoader;
import org.columba.mail.config.MailConfig;
import org.columba.mail.config.OutgoingItem;
import org.columba.mail.folder.AbstractFolder;
import org.columba.mail.folder.virtual.ActivateVirtualFolderCommand;
import org.columba.mail.gui.composer.ComposerController;
import org.columba.mail.gui.composer.ComposerModel;
import org.columba.mail.gui.config.accountwizard.AccountWizardLauncher;
import org.columba.mail.gui.tree.FolderTreeModel;
import org.columba.mail.nativ.defaultmailclient.SystemDefaultMailClientHandler;
import org.columba.mail.parser.MailUrlParser;
import org.columba.mail.pgp.MultipartEncryptedRenderer;
import org.columba.mail.pgp.MultipartSignedRenderer;
import org.columba.mail.shutdown.SaveAllFoldersPlugin;
import org.columba.mail.shutdown.SavePOP3CachePlugin;
import org.columba.mail.spam.SaveSpamDBPlugin;
import org.columba.mail.util.MailResourceLoader;
import org.columba.ristretto.composer.MimeTreeRenderer;
import org.columba.ristretto.parser.ParserException;

/**
 * Main entrypoint for mail component.
 * 
 * @author fdietz
 */
public class MailMain implements IComponentPlugin {
	/** JDK 1.4+ logging framework logger, used for logging. */
	private static final Logger LOG = Logger.getLogger("org.columba.mail.main");

	private static final String RESOURCE_PATH = "org.columba.mail.i18n.global";

	public MailMain() {
	}

	/**
	 *  
	 */
	public void init() {
		// Init PGP
		MimeTreeRenderer renderer = MimeTreeRenderer.getInstance();
		renderer.addMimePartRenderer(new MultipartSignedRenderer());
		renderer.addMimePartRenderer(new MultipartEncryptedRenderer());

		// Init Plugins
		PluginManager.getInstance().addHandlers(
				"org/columba/mail/plugin/pluginhandler.xml");

		try {
			((ActionPluginHandler) PluginManager.getInstance().getHandler(
					"org.columba.core.action"))
					.addActionList("org/columba/mail/action/action.xml");
		} catch (PluginHandlerNotFoundException ex) {
			ex.printStackTrace();
		}

		TaskInterface plugin = new SaveAllFoldersPlugin();
		BackgroundTaskManager.getInstance().register(plugin);

		plugin = new SavePOP3CachePlugin();
		BackgroundTaskManager.getInstance().register(plugin);
		ShutdownManager.getShutdownManager().register(plugin);

		plugin = new SaveSpamDBPlugin();
		BackgroundTaskManager.getInstance().register(plugin);
		ShutdownManager.getShutdownManager().register(plugin);

		ServiceManager.getInstance().register(
				"org.columba.mail.facade.IConfigFactory",
				"org.columba.mail.facade.ConfigFactory");
		ServiceManager.getInstance().register(
				"org.columba.mail.facade.IComposerFactory",
				"org.columba.mail.facade.ComposerFactory");
		ServiceManager.getInstance().register(
				"org.columba.mail.facade.IDialogFactory",
				"org.columba.mail.facade.DialogFactory");
		ServiceManager.getInstance().register(
				"org.columba.mail.facade.IFolderFactory",
				"org.columba.mail.facade.FolderFactory");
		ServiceManager.getInstance().register(
				"org.columba.mail.facade.ISelectionFactory",
				"org.columba.mail.facade.SelectionFactory");
	}

	public void registerCommandLineArguments() {
		ColumbaCmdLineParser parser = ColumbaCmdLineParser.getInstance();

		parser.addOption(OptionBuilder.withArgName("options").hasArg()
				.withDescription(
						GlobalResourceLoader.getString(RESOURCE_PATH, "global",
								"cmdline_composer")).create("compose"));

		Option mailOption = OptionBuilder.withArgName("mailurl").hasArg()
				.withDescription(
						GlobalResourceLoader.getString(RESOURCE_PATH, "global",
								"cmdline_mail_withurl")).create("mail");

		mailOption.setOptionalArg(true);

		parser.addOption(mailOption);
	}

	/**
	 * @see org.columba.core.main.IComponentPlugin#handleCommandLineParameters(java.lang.String[])
	 */
	public void handleCommandLineParameters(CommandLine commandLine) {
		if (commandLine==null) return;
		if (commandLine.hasOption("mail")) {

			if (commandLine.getOptionValue("mail") != null) {
				try {
					ComposerModel model = new ComposerModel(MailUrlParser
							.parse(commandLine.getOptionValue("mail")));

					//new NewMessageAction().actionPerformed(null);
					ComposerController controller = new ComposerController();
					new DefaultContainer(controller);

					controller.setComposerModel(model);

					Main.getInstance().setRestoreLastSession(false);
				} catch (ParserException e1) {
					LOG.warning(e1.getLocalizedMessage());
				}
			} else {
				try {
					FrameModel.getInstance().openView("ThreePaneMail");

					Main.getInstance().setRestoreLastSession(false);
				} catch (PluginLoadingFailedException e) {
					LOG.severe(e.getLocalizedMessage());
				}
			}
		}

		if (commandLine.hasOption("compose")) {
			ComposerModel model = new ComposerModel(MessageOptionParser
					.parse(commandLine.getOptionValue("compose")));

			//new NewMessageAction().actionPerformed(null);
			ComposerController controller = new ComposerController();
			new DefaultContainer(controller);

			controller.setComposerModel(model);

			Main.getInstance().setRestoreLastSession(false);
		}
	}

	/**
	 *  
	 */
	public void postStartup() {
		// Check default mail client
		checkDefaultClient();

		// Show first time Account Wizard
		if (MailConfig.getInstance().getAccountList().count() == 0) {
			new AccountWizardLauncher().launchWizard(true);
		}

		// Check Internet Connection
		if( MailConfig.getInstance().getAccountList().count() > 0 ) {
			OutgoingItem testConnection = MailConfig.getInstance().getAccountList().get(0).getSmtpItem();
			ConnectionStateImpl.getInstance().setTestConnection(testConnection.get("host"), testConnection.getInteger("port"));
			ConnectionStateImpl.getInstance().checkPhysicalState();
		}
		
		
		// Activate all Virtual Folders
		ActivateVirtualFolderCommand.activateAll((AbstractFolder)FolderTreeModel.getInstance().getRoot());
	}

	private void checkDefaultClient() {
		// Check if Columba is the default mail client
		SystemDefaultMailClientHandler defaultClientHandler = new SystemDefaultMailClientHandler();
		IDefaultItem item = new DefaultItem(MailConfig.getInstance().get(
				"options"));

		boolean checkDefault = item.getBooleanWithDefault("options/defaultclient",
				"check", true);

		if (checkDefault
				&& defaultClientHandler.platfromSupportsDefaultMailClient()) {
			if (!defaultClientHandler.isDefaultMailClient()) {

				JPanel panel = new JPanel(new BorderLayout(0, 10));

				panel.add(new MultiLineLabel(MailResourceLoader.getString(
						"dialog", "defaultclient", "make_default")),
						BorderLayout.NORTH);

				JCheckBox askAgain = new JCheckBox(MailResourceLoader
						.getString("dialog", "defaultclient", "ask_no_more"));
				panel.add(askAgain, BorderLayout.CENTER);

				// Some error in the client/server communication
				//  --> fall back to default login process
				int result = JOptionPane.showConfirmDialog(FrameModel
						.getInstance().getActiveFrame(), panel,
						MailResourceLoader.getString("dialog", "defaultclient",
								"title"), JOptionPane.YES_NO_OPTION);
				if (result == JOptionPane.OK_OPTION) {
					defaultClientHandler.setDefaultMailClient();
				}

				item.setBoolean("options/defaultclient", "check", !askAgain
						.isSelected());
			}
		}
	}
}